@@ -114,7 +114,7 @@ impl Pyxel {
         for i in 0..NUM_CHANNELS {
             self.play(
                 i,
-                &music.lock().sequences[i as usize],
+                &music.lock().sounds_list[i as usize],
                 start_tick,
                 should_loop,
             );
