@@ -2,8 +2,8 @@ use crate::key::KEY_ESCAPE;
 use crate::types::{Color, Effect, Key, Note, Rgb8, Speed, Tone, Volume};
 
 // System
-pub const PYXEL_VERSION: &str = "1.6.8";
-pub const PYXEL_WORKING_DIR: &str = "~/.pyxel";
+pub const PYXEL_VERSION: &str = "1.7.0";
+pub const PYXEL_WORKING_DIR: &str = ".pyxel";
 pub const DEFAULT_TITLE: &str = "Pyxel";
 pub const DEFAULT_FPS: u32 = 30;
 pub const DEFAULT_QUIT_KEY: Key = KEY_ESCAPE;
